################################################################################
# EPIRICE Run Created by Kwang-Hyung Kim (15 Jan. 2015)
################################################################################

#REQUIRED PACKAGES
library(cropsim)

# RUN EPIRICE FOR SHEATH BLIGHT AND LEAF BLAST
# Weather data and rice transplanting date are to be provide seperated following the original EPIRICE coding format (Savary et al. 2012) 
# wth = weather data (same as the original EPIRICE wth format); tpd = rice transplaning date (yyyy-mm-dd, provided seperately)

AUDPC_SB = Cal.SB.audpc(wth, tpd) 
AUDPC_LB = Cal.LB.audpc(wth, tpd)



################################################################################
################################################################################
### Modified FUNCTIONS ###

################################################################################
# EPIRICE SEIR.SB Function -> derived from SEIR function from original EPIRICE
################################################################################
SEIR.SB <- function (wth, tpd, onset, duration = 100, rhlim = 95, rainlim = 5, wetness, 
                     initSites, initInfection = 1, ageRc, baseRc, tmpRc, rhRc, latrans, 
                     inftrans, siteMax, AGGR, RRPhysiolSenesc, RRG, SenescType = 1) {
  
  tpd <- as.Date(tpd)
  wth@w <- subset(wth@w, wth@w$date >= tpd - 1)
  
  if (dim(wth@w)[1] < duration) {
    stop("Incomplete weather data")
  }
  wth@w <- wth@w[1:(duration + 1), ]
  if (wetness == 1) {
    W <- leafWet(wth, simple = TRUE)
  }
  COFR <- Rc <- RHCoef <- latency <- infectious <- Severity <- RSenesced <- RGrowth <- Rtransfer <- Rinfection <- Diseased <- Senesced <- Removed <- now_infectious <- now_latent <- Sites <- TotalSites <- rep(0, 
                                                                                                                                                                                                                times = duration + 1)
  for (day in 0:duration) {
    if (day == 0) {
      Sites[day + 1] <- initSites
      RSenesced[day + 1] <- RRPhysiolSenesc * Sites[day + 1]
    } else {
      if (day > inftrans) {
        removedToday <- infectious[infday + 2]
      } else {
        removedToday <- 0
      }
      Sites[day + 1] <- Sites[day] + RGrowth[day] - Rinfection[day] - 
        RSenesced[day]
      RSenesced[day + 1] <- removedToday * SenescType + 
        RRPhysiolSenesc * Sites[day + 1]
      Senesced[day + 1] <- Senesced[day] + RSenesced[day]
      latency[day + 1] <- Rinfection[day]
      latday <- day - latrans + 1
      latday <- max(0, latday)
      now_latent[day + 1] <- sum(latency[latday:day + 1])
      infectious[day + 1] <- Rtransfer[day]
      infday <- day - inftrans + 1
      infday <- max(0, infday)
      now_infectious[day + 1] <- sum(infectious[infday:day + 1])
    }
    if (Sites[day + 1] < 0) {
      Sites[day + 1] <- 0
      break
    }
    if (wetness == 0) {
      if (wth@w$rhmax[day + 1] >= rhlim | wth@w$prcp[day + 1] >= rainlim) {
        RHCoef[day + 1] <- 1
      } else {
        W <- leafWet(wth, simple = TRUE)
        RHCoef[day + 1] <- AFGen(rhRc, W[day + 1])
      }
    }
    Rc[day + 1] <- baseRc * AFGen(ageRc, day) * AFGen(tmpRc, wth@w$tavg[day + 1]) * RHCoef[day + 1]
    Diseased[day + 1] <- sum(infectious) + now_latent[day + 1]+ Removed[day + 1] 
    Removed[day + 1] <- sum(infectious) - now_infectious[day + 1]
    COFR[day + 1] <- 1 - (Diseased[day + 1]/(Sites[day + 1] + Diseased[day + 1]))
    if (day == onset) {
      Rinfection[day + 1] <- initInfection
    } else if (day > onset) {
      Rinfection[day + 1] <- now_infectious[day + 1] * Rc[day + 1] * (COFR[day + 1]^AGGR)
    } else {
      Rinfection[day + 1] <- 0
    }
    if (day >= latrans) {
      Rtransfer[day + 1] <- latency[latday + 1]
    } else {
      Rtransfer[day + 1] <- 0
    }
    TotalSites[day + 1] <- Diseased[day + 1] + Sites[day + 1]
    RGrowth[day + 1] <- AFGen(RRG, day) * Sites[day + 1] * (1 - (TotalSites[day + 1]/siteMax))
    Severity[day + 1] <- (Diseased[day + 1] - Removed[day + 1])/(TotalSites[day + 1] - Removed[day + 1]) * 100
  }
  res <- cbind(0:duration, TotalSites, Sites, now_latent, now_infectious, 
               Removed, Senesced, Rinfection, Rtransfer, RGrowth, RSenesced, 
               Diseased, Severity)
  res <- as.data.frame(res[1:(day + 1), ])
  dates <- seq(tpd - 1, tpd + duration, 1)
  res <- cbind(dates[1:(day + 1)], res)
  colnames(res) <- c("date", "simday", "tsites", "sites", "latent", "infectious", 
                     "removed", "senesced", "rateinf", "rtransfer", "rgrowth", 
                     "rsenesced", "diseased", "severity")
  result <- new("SEIR")
  result@d <- res
  return(result)
}



################################################################################
# EPIRICE SEIR.LB Function -> derived from SEIR function from original EPIRICE
################################################################################
SEIR.LB <- function (wth, tpd, onset, duration = 100, rhlim = 90, rainlim = 5, wetness, 
                     initSites, initInfection = 1, ageRc, baseRc, tmpRc, rhRc, latrans,
                     inftrans, siteMax, AGGR, RRPhysiolSenesc, RRG, SenescType = 1) {
  
  tpd <- as.Date(tpd)
  wth@w <- subset(wth@w, wth@w$date >= tpd - 1)
  
  if (dim(wth@w)[1] < duration) {
    stop("Incomplete weather data")
  }
  wth@w <- wth@w[1:(duration + 1), ]
  if (wetness == 1) {
    W <- leafWet(wth, simple = TRUE)
  }
  COFR <- Rc <- RHCoef <- latency <- infectious <- Severity <- RSenesced <- RGrowth <- Rtransfer <- Rinfection <- Diseased <- Senesced <- Removed <- now_infectious <- now_latent <- Sites <- TotalSites <- rep(0, 
                                                                                                                                                                                                                times = duration + 1)
  for (day in 0:duration) {
    if (day == 0) {
      Sites[day + 1] <- initSites
      RSenesced[day + 1] <- RRPhysiolSenesc * Sites[day + 1]
    } else {
      if (day > inftrans) {
        removedToday <- infectious[infday + 2]
      } else {
        removedToday <- 0
      }
      Sites[day + 1] <- Sites[day] + RGrowth[day] - Rinfection[day] - 
        RSenesced[day]
      RSenesced[day + 1] <- removedToday * SenescType + 
        RRPhysiolSenesc * Sites[day + 1]
      Senesced[day + 1] <- Senesced[day] + RSenesced[day]
      latency[day + 1] <- Rinfection[day]
      latday <- day - latrans + 1
      latday <- max(0, latday)
      now_latent[day + 1] <- sum(latency[latday:day + 1])
      infectious[day + 1] <- Rtransfer[day]
      infday <- day - inftrans + 1
      infday <- max(0, infday)
      now_infectious[day + 1] <- sum(infectious[infday:day + 1])
    }
    if (Sites[day + 1] < 0) {
      Sites[day + 1] <- 0
      break
    }
    if (wetness == 0) {
      if (wth@w$rhmax[day + 1] >= rhlim | wth@w$prcp[day + 1] >= rainlim) {
        RHCoef[day + 1] <- 1
      } else {
        W <- leafWet(wth, simple = TRUE)
        RHCoef[day + 1] <- AFGen(rhRc, W[day + 1])
      }
    }
    Rc[day + 1] <- baseRc * AFGen(ageRc, day) * AFGen(tmpRc, wth@w$tavg[day + 1]) * RHCoef[day + 1]
    Diseased[day + 1] <- sum(infectious) + now_latent[day + 1]+ Removed[day + 1] 
    Removed[day + 1] <- sum(infectious) - now_infectious[day + 1]
    COFR[day + 1] <- 1 - (Diseased[day + 1]/(Sites[day + 1] + Diseased[day + 1]))
    if (day == onset) {
      Rinfection[day + 1] <- initInfection
    } else if (day > onset) {
      Rinfection[day + 1] <- now_infectious[day + 1] * Rc[day + 1] * (COFR[day + 1]^AGGR)
    } else {
      Rinfection[day + 1] <- 0
    }
    if (day >= latrans) {
      Rtransfer[day + 1] <- latency[latday + 1]
    } else {
      Rtransfer[day + 1] <- 0
    }
    TotalSites[day + 1] <- Diseased[day + 1] + Sites[day + 1]
    RGrowth[day + 1] <- AFGen(RRG, day) * Sites[day + 1] * (1 - (TotalSites[day + 1]/siteMax))
    Severity[day + 1] <- (Diseased[day + 1] - Removed[day + 1])/(TotalSites[day + 1] - Removed[day + 1]) * 100
  }
  res <- cbind(0:duration, TotalSites, Sites, now_latent, now_infectious, 
               Removed, Senesced, Rinfection, Rtransfer, RGrowth, RSenesced, 
               Diseased, Severity)
  res <- as.data.frame(res[1:(day + 1), ])
  dates <- seq(tpd - 1, tpd + duration, 1)
  res <- cbind(dates[1:(day + 1)], res)
  colnames(res) <- c("date", "simday", "tsites", "sites", "latent", "infectious", 
                     "removed", "senesced", "rateinf", "rtransfer", "rgrowth", 
                     "rsenesced", "diseased", "severity")
  result <- new("SEIR")
  result@d <- res
  return(result)
}


################################################################################
# EPIRICE-SB Function -> derived from sheathBlight function from original EPIRICE
################################################################################

sheathBlight.EPIRICE <- function (wth, tpd, ...) {
  AgeCoefRc <- cbind(0:12 * 10, c(0.43, 0.5, 0.73, 0.81, 0.94,
                                  1, 1, 1, 1, 0.01, 0.8, 0.7, 0.5)) 
  RHCoefRc <- cbind(c(0:10), c(0, 0.24, 0.41, 0.68, 0.94, 
                               0.97, 1,1,1,1,1))
  TempCoefRc <- cbind(4:13 * 3, c(0, 0.1, 0.8, 1, 1, 1, 0.2, 
                                  0.01, 0.01, 0))   
  RRPhysiolGrowth <- cbind(0:12 * 10, c(0.15, 0.21, 0.11, 0.0001, 0.0001, 0.001, 
                                        0.005, 0.005,0.005,0.005,0.005,0.005,0.005))
  return(SEIR.SB(wth = wth, tpd = tpd, onset = 30, ageRc = AgeCoefRc, 
                 tmpRc = TempCoefRc, rhRc = RHCoefRc, baseRc = 0.58, latrans = 4, 
                 inftrans = 65, siteMax = 800, AGGR = 2.8, initSites = 90, 
                 RRPhysiolSenesc = 0.005, RRG = RRPhysiolGrowth, wetness = 0))
}


################################################################################
# EPIRICE-LB Function -> derived from leafBlast function from original EPIRICE
################################################################################

leafBlast.EPIRICE <- function (wth, tpd, ...) {
  
  AgeCoefRc <- cbind(0:24 * 5, c(1, 1, 1, 1, 1, 1, 1, 1, 1, 0.4, 0.3,  
                                 0.1, 0.1, 0.05, 0.03, 0.03, 0.02, 0.02, 0.02, 
                                 0.01, 0.01, 0.01, 0.01, 0.01, 0.01))
  TempCoefRc <- cbind(3:13 * 3, c(0, 0.4, 0.72, 0.88, 1, 0.9, 0.45,
                                  0.2, 0.05, 0.01, 0))
  RHCoefRc <- cbind(c(0:10), c(0, 0.24, 0.41, 0.68, 0.94, 
                               0.97, 1,1,1,1,1))
  RRPhysiolGrowth <- cbind(0:12 * 10, c(0.12, 0.12, 0.11, 0.11, 0.09, 0.09, 
                                        0.09, 0.09, 0.09, 0.09, 0.05, 0.01, 0.0001))
  return(SEIR.LB(wth = wth, tpd = tpd, onset = 15, ageRc = AgeCoefRc, 
                 tmpRc = TempCoefRc, rhRc = RHCoefRc, baseRc = 0.86, latrans = 4, 
                 inftrans = 20, initSites = 600, AGGR = 1, siteMax = 90000, 
                 RRPhysiolSenesc = 0.005, RRG = RRPhysiolGrowth, wetness = 0,...))
}


################################################################################
# EPIRICE-SB audpc calculation Function
################################################################################
Cal.SB.audpc <- function(wth, tpd) {
    
  shblight <- sheathBlight.EPIRICE(wth, tpd)
  
  if(class(shblight) != "try-error"){
    shblgtout <- sum(shblight@d$severity[1:100])    
  } else {
    shblgtout <- -999
  }
  names(shblgtout) <- "SB_audpc"
  return(shblgtout)
}


################################################################################
# EPIRICE-LB audpc calculation Function
################################################################################
Cal.LB.audpc <- function(wth, tpd) {
  
  lfblast <- leafBlast.EPIRICE(wth, tpd)
  
  if(class(lfblast) != "try-error"){
    lfblstout <- sum(lfblast@d$severity[1:100])    
  } else {
    lfblstout <- -999
  }
  names(lfblstout) <- "LB_audpc"
  return(lfblstout)
}

